@@ -135,15 +135,18 @@ def translateErrorCode(ClientID :int) -> str:
         
         If there is no match, returns the clientID as str.
         """
-        if 0 <= ClientID < 128:
-            return "NO_ERRORS"
+        if isinstance(ClientID, str): # if this got passed a string, return the string
+            return ClientID
         if ClientID < 0: # some functions return negative value for error code
             ClientID *= -1
+        ClientID &= 0xFFFF # Noregon can add garbage to leading bytes
         if ClientID > 0x8000:
             ClientID = 0xFFFF - ClientID
+        if 0 <= ClientID < 128:
+            return "NO_ERRORS"
         return RP1210_ERRORS.get(ClientID, str(ClientID))
 
-def getAPINames(rp121032_path = None) -> list[str]:
+def getAPINames(rp121032_path : str = None) -> list[str]:
     """
     A function for reading API names from RP121032.ini. Returns a list of strings.
 
@@ -156,12 +159,12 @@ def getAPINames(rp121032_path = None) -> list[str]:
     """
     if not rp121032_path: # find our own path if none is given
         rp121032_path = os.path.join(os.environ["WINDIR"], "RP121032.ini")
-    if not os.path.isfile(rp121032_path): # check if file exists
-        return []
-    try:    # read from file
+    elif not os.path.isfile(rp121032_path): # check if file exists
+        raise FileNotFoundError(f"RP121032.ini not found at {rp121032_path}.")
+    try:
         parser = ConfigParser()
         parser.read(rp121032_path)
-        return parser["RP1210Support"]["APIImplementations"].split(",")
+        return parser.get("RP1210Support", "APIImplementations").split(",")
     except Exception:
         return []
 
@@ -194,11 +197,7 @@ class RP1210Protocol:
     def getSpeed(self) -> list[str]:
         """Returns ProtocolSpeed parameters as a list of strings."""
         try:
-            speeds = []
-            section_list = str(self.section["ProtocolSpeed"]).split(',')
-            for speed in section_list:
-                speeds.append(speed)
-            return speeds
+            return str(self.section["ProtocolSpeed"]).split(',')
         except Exception:
             return []
 
@@ -227,6 +226,11 @@ class RP1210Protocol:
         except Exception:
             return []
 
+    def __eq__(self, __o: object) -> bool:
+        if not isinstance(__o, RP1210Protocol):
+            raise TypeError("Tried to compare RP1210Protocol with innappropriate object type.")
+        return self.section == __o.section
+
 class RP1210Device:
     """
     Stores information for an RP1210 device, e.g. info stored in DeviceInformationXXX sections.
@@ -244,7 +248,11 @@ class RP1210Device:
         self.section = section
 
     def getID(self) -> int:
-        """Returns DeviceID parameter as int."""
+        """
+        Returns DeviceID parameter as int.
+        
+        Returns -1 if DeviceID is invalid.
+        """
         try:
             return int(self.section["DeviceID"])
         except Exception:
@@ -295,6 +303,11 @@ class RP1210Device:
         if self.getDescription() != "":
             ret_str += " - " + self.getDescription()
         return ret_str
+
+    def __eq__(self, __o: object) -> bool:
+        if not isinstance(__o, RP1210Device):
+            raise TypeError("Tried to compare RP1210Device with innappropriate object type.")
+        return self.section == __o.section
          
 class RP1210Config(ConfigParser):
     """
@@ -310,11 +323,12 @@ class RP1210Config(ConfigParser):
 
     You can use str(this_object) to generate a string to display in your Vendors dropdown.
     """
-    def __init__(self, api_name : str) -> None:
+    def __init__(self, api_name : str, api_path : str = None, config_path : str = None) -> None:
         super().__init__()
         self._api_name = api_name
         self._api_valid = True
-        self.api = RP1210API(api_name)
+        self.api = RP1210API(api_name, api_path)
+        self._configDir = config_path
         self.populate()
 
     def __str__(self) -> str:
@@ -359,9 +373,7 @@ class RP1210Config(ConfigParser):
 
         Will return "(Vendor Name Missing)" if the 'Name' field isn't found.
         """
-        if not self.has_option("VendorInformation", "Name"):
-            return "(Vendor Name Missing)"
-        return self.get("VendorInformation", "Name")
+        return self.get("VendorInformation", "Name", fallback="(Vendor Name Missing)")
 
     def getDescription(self) -> str:
         """
@@ -380,9 +392,7 @@ class RP1210Config(ConfigParser):
 
         Returns an empty string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "Address1"):
-            return ""
-        return self.get("VendorInformation", "Address1")
+        return self.get("VendorInformation", "Address1", fallback="")
 
     def getAddress2(self) -> str:
         """
@@ -390,9 +400,7 @@ class RP1210Config(ConfigParser):
 
         Returns an empty string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "Address2"):
-            return ""
-        return self.get("VendorInformation", "Address2")
+        return self.get("VendorInformation", "Address2", fallback="")
 
     def getCity(self) -> str:
         """
@@ -400,9 +408,7 @@ class RP1210Config(ConfigParser):
 
         Returns an empty string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "City"):
-            return ""
-        return self.get("VendorInformation", "City")
+        return self.get("VendorInformation", "City", fallback="")
 
     def getState(self) -> str:
         """
@@ -410,9 +416,7 @@ class RP1210Config(ConfigParser):
 
         Returns an empty string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "State"):
-            return ""
-        return self.get("VendorInformation", "State")
+        return self.get("VendorInformation", "State", fallback="")
 
     def getCountry(self) -> str:
         """
@@ -420,9 +424,7 @@ class RP1210Config(ConfigParser):
 
         Returns an empty string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "Country"):
-            return ""
-        return self.get("VendorInformation", "Country")
+        return self.get("VendorInformation", "Country", fallback="")
 
     def getPostal(self) -> str:
         """
@@ -430,9 +432,7 @@ class RP1210Config(ConfigParser):
 
         Returns an empty string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "Postal"):
-            return ""
-        return self.get("VendorInformation", "Postal")
+        return self.get("VendorInformation", "Postal", fallback="")
 
     def getTelephone(self) -> str:
         """
@@ -440,9 +440,7 @@ class RP1210Config(ConfigParser):
 
         Returns an empty string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "Telephone"):
-            return ""
-        return self.get("VendorInformation", "Telephone")
+        return self.get("VendorInformation", "Telephone", fallback="")
 
     def getFax(self) -> str:
         """
@@ -450,9 +448,7 @@ class RP1210Config(ConfigParser):
 
         Returns an empty string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "Fax"):
-            return ""
-        return self.get("VendorInformation", "Fax")
+        return self.get("VendorInformation", "Fax", fallback="")
 
     def getVendorURL(self) -> str:
         """
@@ -460,9 +456,7 @@ class RP1210Config(ConfigParser):
         
         Returns empty string if VendorURL field isn't found.
         """
-        if not self.has_option("VendorInformation", "VendorURL"):
-            return ""
-        return self.get("VendorInformation", "VendorURL")
+        return self.get("VendorInformation", "VendorURL", fallback="")
 
     def getVersion(self) -> str:
         """
@@ -470,12 +464,7 @@ class RP1210Config(ConfigParser):
         
         Returns empty string if Version field isn't found.
         """
-        if not self.has_option("VendorInformation", "Version"):
-            return ""
-        try:
-            return self.get("VendorInformation", "Version")
-        except (ValueError, KeyError):
-            return ""
+        return self.get("VendorInformation", "Version", fallback="")
 
     def getAutoDetectCapable(self) -> bool:
         """
@@ -483,10 +472,8 @@ class RP1210Config(ConfigParser):
 
         Returns False if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "AutoDetectCapable"):
-            return False
         try:
-            return self.getboolean("VendorInformation", "AutoDetectCapable")
+            return self.getboolean("VendorInformation", "AutoDetectCapable", fallback=False)
         except (ValueError, KeyError):
             return False
 
@@ -504,14 +491,12 @@ class RP1210Config(ConfigParser):
         """
         Returns the 'TimeStampWeight' field in VendorInformation section.
 
-        Returns None if the field isn't found.
+        Returns 1.0 if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "TimeStampWeight"):
-            return None
         try:
-            return self.getfloat("VendorInformation", "TimeStampWeight")
+            return self.getfloat("VendorInformation", "TimeStampWeight", fallback=1.0)
         except (ValueError, KeyError):
-            return None
+            return 1.0
 
     def getMessageString(self) -> str:
         """
@@ -519,9 +504,7 @@ class RP1210Config(ConfigParser):
 
         Returns a blank string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "MessageString"):
-            return ""
-        return self.get("VendorInformation", "MessageString")
+        return self.get("VendorInformation", "MessageString", fallback="")
 
     def getErrorString(self) -> str:
         """
@@ -529,9 +512,7 @@ class RP1210Config(ConfigParser):
 
         Returns a blank string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "ErrorString"):
-            return ""
-        return self.get("VendorInformation", "ErrorString")
+        return self.get("VendorInformation", "ErrorString", fallback="")
 
     def getRP1210Version(self) -> str:
         """
@@ -539,9 +520,7 @@ class RP1210Config(ConfigParser):
 
         Returns a blank string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "RP1210"):
-            return ""
-        return self.get("VendorInformation", "RP1210")
+        return self.get("VendorInformation", "RP1210", fallback="")
 
     def getDebugLevel(self) -> int:
         """
@@ -555,10 +534,8 @@ class RP1210Config(ConfigParser):
 
         Returns -1 (debugging not supported) if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "DebugLevel"):
-            return -1
         try:
-            return self.getint("VendorInformation", "DebugLevel")
+            return self.getint("VendorInformation", "DebugLevel", fallback=-1)
         except (ValueError, KeyError):
             return -1
 
@@ -570,9 +547,7 @@ class RP1210Config(ConfigParser):
 
         Returns a blank string if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "DebugFile"):
-            return ""
-        return self.get("VendorInformation", "DebugFile")
+        return self.get("VendorInformation", "DebugFile", fallback="")
 
     def getDebugMode(self) -> int:
         """
@@ -580,14 +555,12 @@ class RP1210Config(ConfigParser):
         - 0 = Overwrite (completely destroying previous contents) 
         - 1 = Append (write to the end of the file, keeping any previous contents) 
 
-        Returns None if the field isn't found.
+        Returns -1 if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "DebugMode"):
-            return None
         try:
-            return self.getint("VendorInformation", "DebugMode")
+            return self.getint("VendorInformation", "DebugMode", fallback=-1)
         except (ValueError, KeyError):
-            return None
+            return -1
 
     def getDebugFileSize(self) -> int:
         """
@@ -599,10 +572,8 @@ class RP1210Config(ConfigParser):
         Returns 1024 (default size) if the field isn't found. Please note that if DebugLevel = -1,
         there will be no logging even if you receive a value of 1024 from this function.
         """
-        if not self.has_option("VendorInformation", "DebugFileSize"):
-            return 1024
         try:
-            return self.getint("VendorInformation", "DebugFileSize")
+            return self.getint("VendorInformation", "DebugFileSize", fallback=1024)
         except (ValueError, KeyError):
             return 1024
 
@@ -615,10 +586,8 @@ class RP1210Config(ConfigParser):
 
         Returns 1 (default value) if the field isn't found.
         """
-        if not self.has_option("VendorInformation", "NumberOfRTSCTSSessions"):
-            return 1
         try:
-            return self.getint("VendorInformation", "NumberOfRTSCTSSessions")
+            return self.getint("VendorInformation", "NumberOfRTSCTSSessions", fallback=1)
         except (ValueError, KeyError):
             return 1
 
@@ -634,10 +603,7 @@ class RP1210Config(ConfigParser):
         if not self.has_option("VendorInformation", "CANFormatsSupported"):
             return []
         try:
-            params = []
-            for param in self["VendorInformation"]["CANFormatsSupported"].split(","):
-                params.append(int(param))
-            return params
+            return [int(i) for i in self["VendorInformation"]["CANFormatsSupported"].split(",")]
         except Exception:
             return []
 
@@ -662,10 +628,8 @@ class RP1210Config(ConfigParser):
 
     def getCANAutoBaud(self) -> bool:
         """Returns the CANAutoBaud field in VendorInformation."""
-        if not self.has_option("VendorInformation", "CANAutoBaud"):
-            return False
         try:
-            return self.getboolean("VendorInformation", "CANAutoBaud")
+            return self.getboolean("VendorInformation", "CANAutoBaud", fallback=False)
         except (ValueError, KeyError):
             return False
 
@@ -684,8 +648,7 @@ class RP1210Config(ConfigParser):
         Returns None if the Device isn't found.
         """
         try:
-            section = self["DeviceInformation" + str(deviceID)]
-            return RP1210Device(section)
+            return RP1210Device(self["DeviceInformation" + str(deviceID)])
         except Exception:
             return None
 
@@ -696,8 +659,8 @@ class RP1210Config(ConfigParser):
         try:
             deviceList = [] #type: list[RP1210Device]
             deviceIDs = self.getDeviceIDs()
-            for id in deviceIDs:
-                section = self["DeviceInformation" + str(id)]
+            for device_num in deviceIDs:
+                section = self["DeviceInformation" + str(device_num)]
                 deviceList.append(RP1210Device(section))
             return deviceList
         except Exception:
@@ -757,12 +720,12 @@ class RP1210Config(ConfigParser):
         Returns [] if no protocols are found.
         """
         try:
-            ids = self.getProtocolIDs()
-            if not ids:
+            protocol_ids = self.getProtocolIDs()
+            if not protocol_ids:
                 return []
             strings = []
-            for id in ids:
-                strings.append(self.getProtocol(id).getString())
+            for protocol_num in protocol_ids:
+                strings.append(self.getProtocol(protocol_num).getString())
             return strings
         except Exception:
             return []
@@ -770,15 +733,15 @@ class RP1210Config(ConfigParser):
     def getProtocolIDs(self) -> list[int]:
         """Returns list of ProtocolIDs described in .ini file."""
         try:
-            protocols = []
-            for protocol in self["VendorInformation"]["Protocols"].split(","):
-                protocols.append(int(protocol))
-            return protocols
+            if not self.has_option("VendorInformation", "Protocols"):
+                return []
+            return [int(i) for i in self["VendorInformation"]["Protocols"].split(",")]
         except Exception:
             return []
 
     def populate(self):
         """Reads .ini file for the specified RP1210 API."""
+        
         try:
             path = self.getPath()
             if not os.path.exists(path):
@@ -791,7 +754,27 @@ class RP1210Config(ConfigParser):
 
     def getPath(self) -> str:
         """Returns absolute path to API config file."""
-        return os.path.join(os.environ["WINDIR"], self._api_name + ".ini")
+       
+        if self._configDir is not None:
+            if os.path.abspath(self._configDir):
+
+                if os.path.isfile(self._configDir):
+                    # [provided absolute path]
+                    return self._configDir
+                else:
+                    # [provided absolute dir] + [api name].ini
+                    return os.path.join(self._configDir, self._api_name + ".ini")
+
+            else:
+                if os.path.isfile(os.path.join(os.curdir, self._configDir)):
+                    # [current directory] + [provided relative path]
+                    return os.path.join(os.curdir, self._configDir)
+                else:
+                    return os.path.join(os.curdir, self._configDir + self._api_name + ".ini")
+                    # [current directory] + [provided relative dir] + [api name].ini
+                
+        else:
+            return os.path.join(os.environ["WINDIR"], self._api_name + ".ini")
 
 class RP1210API:
     """
@@ -799,11 +782,16 @@ class RP1210API:
 
     See function docstrings for details on each function.
     """
-    def __init__(self, api_name : str) -> None:
+    def __init__(self, api_name : str, WorkingAPIDirectory : str = None) -> None:
         self._api_valid = False
         self._api_name = api_name
         self.dll = None
         self._conforms_to_rp1210c = True
+        self._libDir = WorkingAPIDirectory
+
+    def getAPIName(self) -> str:
+        """Returns API name for this API."""
+        return self._api_name
 
     def getDLL(self) -> CDLL:
         """
@@ -820,20 +808,45 @@ class RP1210API:
         Loads and returns CDLL for this API.
         
         If you already called loadDLL(), you can call getDLL() to get the DLL you loaded previously.
+        Can take in a relative and absolute paths files and directories. If given a directory, will attempt to
+        load DLL corresponding to self._api_name from that directory. If a working directory is not provided at
+        initialization of RP1210API(), will assume relative to launch path.
         """
-        try:
+        if self._libDir is not None:
+            path = ""
+            if not os.path.isabs(self._libDir):
+                # If path given is relative, get the working directory
+                if self._libDir is not None:
+                    path += self._libDir
+                else:
+                    path += os.path.abspath(os.curdir)
+
+            path += self._libDir
+                
+            if not os.path.isfile(path):
+                # Append API name to complete path
+                path += self._api_name + ".dll"
             try:
-                path = self._api_name + ".dll"
                 dll = cdll.LoadLibrary(path)
-            except WindowsError:
-                # Try "DLL installed in wrong directory" band-aid
-                path = self.__get_dll_path_aux()
-                dll = cdll.LoadLibrary(path)
-            self.setDLL(dll)
-            return dll
-        except Exception: # RIP
-            self._api_valid = False
-            return None
+                self.setDLL(dll)
+                return dll
+            except Exception: # Couldn't load from input
+                self._api_valid = False
+                return None
+        else:
+            try:
+                try:
+                    path = self._api_name + ".dll"
+                    dll = cdll.LoadLibrary(path)
+                except OSError:
+                    # Try "DLL installed in wrong directory" band-aid
+                    path = self._get_dll_path_aux()
+                    dll = cdll.LoadLibrary(path)
+                self.setDLL(dll)
+                return dll
+            except Exception: # RIP
+                self._api_valid = False
+                return None
 
     def isValid(self) -> bool:
         """
@@ -854,7 +867,6 @@ class RP1210API:
         
         You can do this more easily by reading the RP1210 Version field in RP1210Interface.
         """
-        self.getDLL()
         if not self.isValid():
             return False
         return self._conforms_to_rp1210c
@@ -864,7 +876,7 @@ class RP1210API:
         try:
             self.dll = dll
             if self.dll: # check it's not None
-                self.__init_functions()
+                self._init_functions()
                 self._api_valid = True
             else:
                 self._api_valid = False
@@ -888,7 +900,7 @@ class RP1210API:
         """
         clientID = self.getDLL().RP1210_ClientConnect(0, DeviceID, Protocol, TxBufferSize, 
                                                 RcvBufferSize, isAppPacketizingincomingMsgs)
-        return self.__dla2_clientid_fix(clientID)
+        return self._driver_clientid_fix(clientID)
     
     def ClientDisconnect(self, ClientID : int) -> int:
         """
@@ -897,7 +909,7 @@ class RP1210API:
         Returns 0 if successful, or >127 if it failed.
             You can use translateClientID() to translate the failure code.
         """
-        return self.getDLL().RP1210_ClientDisconnect(ClientID)
+        return self.getDLL().RP1210_ClientDisconnect(ClientID) & 0xFFFF
 
     def SendMessage(self, ClientID : int, ClientMessage : bytes, MessageSize = 0) -> int:
         """
@@ -915,7 +927,7 @@ class RP1210API:
         """
         if MessageSize == 0:
             MessageSize = len(ClientMessage)
-        ret_val = self.getDLL().RP1210_SendMessage(ClientID, ClientMessage, MessageSize, 0, 0)
+        ret_val = self.getDLL().RP1210_SendMessage(ClientID, ClientMessage, MessageSize, 0, 0) & 0xFFFF
         # check for error codes. ret_val is a 16-bit unsigned int, so must be converted
         # to negative signed int.
         if ret_val >= 0x08000:
@@ -939,10 +951,10 @@ class RP1210API:
         """
         if not BufferSize:
             BufferSize = len(RxBuffer)
-        ret_val = self.getDLL().RP1210_ReadMessage(ClientID, RxBuffer, BufferSize, BlockOnRead)
+        ret_val = self.getDLL().RP1210_ReadMessage(ClientID, RxBuffer, BufferSize, BlockOnRead) & 0xFFFF
         # check for error codes. ret_val is a 16-bit unsigned int, so must be converted
         # to negative signed int.
-        if ret_val >= 0x08000:
+        if ret_val >= 0x8000:
             ret_val = (ret_val - 0x10000)
         return ret_val
 
@@ -973,7 +985,7 @@ class RP1210API:
         Usage of ReadVersionDirect() instead of this function is highly recommended.
         """
         return self.getDLL().RP1210_ReadVersion(DLLMajorVersionBuffer, DLLMinorVersionBuffer, 
-                                                APIMajorVersionBuffer, APIMinorVersionBuffer)
+                                                APIMajorVersionBuffer, APIMinorVersionBuffer) & 0xFFFF
 
     def ReadVersionDirect(self, BufferSize = 16) -> tuple:
         """
@@ -1064,7 +1076,7 @@ class RP1210API:
 
         You can also just use GetHardwareStatusDirect() and not worry about buffers.
         """
-        return self.getDLL().RP1210_GetHardwareStatus(ClientID, ClientInfoBuffer, BufferSize, 0)
+        return self.getDLL().RP1210_GetHardwareStatus(ClientID, ClientInfoBuffer, BufferSize, 0) & 0xFFFF
 
 
     def GetHardwareStatusDirect(self, ClientID : int, InfoSize = 64) -> bytes:
@@ -1086,14 +1098,11 @@ class RP1210API:
         
         You really want to read the RP1210C documentation for this one.
         """
-        if MessageSize == 0:
-            if ClientCommand == b"":
-                MessageSize = 0
-            else:
-                MessageSize = len(ClientCommand)
-        return self.getDLL().RP1210_SendCommand(CommandNumber, ClientID, ClientCommand, MessageSize)
+        if MessageSize == 0 and ClientCommand != b"":
+            MessageSize = len(ClientCommand)
+        return self.getDLL().RP1210_SendCommand(CommandNumber, ClientID, ClientCommand, MessageSize) & 0xFFFF
 
-    def __init_functions(self):
+    def _init_functions(self):
         """Give Python type hints for interfacing with the DLL."""
         self.dll.RP1210_ClientConnect.argtypes = [c_long, c_short, c_char_p, c_long, c_long, c_short]
         self.dll.RP1210_ClientDisconnect.argtypes = [c_short]
@@ -1111,26 +1120,30 @@ class RP1210API:
         except Exception: # RP1210C functions not supported
             self._conforms_to_rp1210c = False
 
-    def __get_dll_path_aux(self) -> str:
+    def _get_dll_path_aux(self) -> str:
         """
         Some adapter vendors (looking at you, Actia) install their drivers in the wrong directory.
         This function returns the dll path in that directory.
         """
         return os.path.join(os.environ["WINDIR"], self._api_name + ".dll")
 
-    def __dla2_clientid_fix(self, clientID) -> int:
+    def _driver_clientid_fix(self, clientID) -> int:
         """
         Noregon DLA2 adapters have an issue where they return a bunch of garbage along with the
         ClientID when calling ClientConnect. This is the fix for that.
+
+        PCAN adapters also have an issue when trying to connect to adapters that aren't plugged in,
+        so they get to join the hall of shame.
         """
-        if not self.__is_valid_clientid(clientID):
-            cid = int(hex(clientID)[5:], 16) # snip off first 5 hex characters, translate back to int
-            if self.__is_valid_clientid(cid):
-                clientID = cid
+        if clientID < 0: # some functions return negative value for error code
+            clientID *= -1
+        clientID &= 0xFFFF # Noregon can add garbage to leading bytes
+        if clientID > 0x8000: # catches 2's complement unsigned ints
+            clientID = 0xFFFF - clientID
+        if self._api_name == "PEAKRP32" and clientID > 64:
+            # PCAN drivers give invalid ClientID if it equals 114 (but cover wider range for safety) 
+            clientID = 129  # ERR_INVALID_CLIENT_ID
         return clientID
-    
-    def __is_valid_clientid(self, clientID) -> bool:
-        return clientID in RP1210_ERRORS or (0 <= clientID < 128)
 
 class RP1210VendorList:
     """
@@ -1298,6 +1311,9 @@ class RP1210VendorList:
         except Exception:
             return None
 
+    def getVendorName(self) -> str:
+        return self.getCurrentVendor().getName()
+
     def getCurrentDevice(self) -> RP1210Device:
         """
         Returns RP1210Device object pointed to by device_index.
@@ -1366,7 +1382,7 @@ class RP1210Client(RP1210VendorList):
             if not self.getCurrentVendor().isValid():
                 return 128 # DLL_NOT_INITIALIZED
             deviceID = self.getDeviceID()
-            self.clientID = self.getAPI().ClientConnect(deviceID, protocol)
+            self.clientID = self.getAPI().ClientConnect(deviceID, protocol) & 0xFFFF
             return self.clientID
         except Exception:
             return 128 # DLL_NOT_INITIALIZED
@@ -1379,7 +1395,7 @@ class RP1210Client(RP1210VendorList):
             You can use translateClientID() to translate the failure code.
         """
         try:
-            return self.getAPI().ClientDisconnect(self.clientID)
+            return self.getAPI().ClientDisconnect(self.clientID) & 0xFFFF
         except Exception:
             return 128 # DLL_NOT_INITIALIZED
 
@@ -1390,7 +1406,7 @@ class RP1210Client(RP1210VendorList):
         MessageSize will default to len(ClientCommand) if it is left 0.
         """
         try:
-            return self.getAPI().SendCommand(CommandNumber, self.clientID, ClientCommand, MessageSize)
+            return self.getAPI().SendCommand(CommandNumber, self.clientID, ClientCommand, MessageSize)  & 0xFFFF
         except Exception:
             return -1
 
@@ -1644,6 +1660,17 @@ class RP1210Client(RP1210VendorList):
         """
         return self.command(39)
 
+    def getBaud(self) -> str:
+        """
+        Calls the RP1210_Get_Protocol_Connection_Speed (45) command and returns the value that is
+        received as a string of up to 16 characters.
+        """
+        cmd_num = 45
+        cmd_buffer = create_string_buffer(17)
+        cmd_size = 17
+        self.command(cmd_num, cmd_buffer, cmd_size)
+        return str(cmd_buffer[:cmd_size]) # return first 16 bytes
+
     def setCANBaud(self, baud_code : int, wait_for_msg = True):
         """
         Set CAN Baud Rate (47) (2 bytes)
@@ -1664,5 +1691,4 @@ class RP1210Client(RP1210VendorList):
         cmd_num = 47
         cmd_data = Commands.setCANBaud(baud_code, wait_for_msg)
         cmd_size = 2
-        return self.command(cmd_num, cmd_data, cmd_size)
-
+        return self.command(cmd_num, cmd_data, cmd_size) & 0xFFFF
